- [Your repo](#your-repo)
- [Your syntax](#your-syntax)
- [Your semantics](#your-semantics)
  - [Function sequence](#function-sequence)
  - [Command sequence](#command-sequence)
  - [Operands for arithmetic expressions](#operands-for-arithmetic-expressions)
  - [Operands for boolean expressions](#operands-for-boolean-expressions)
  - [If true](#if-true)
  - [If false](#if-false)
  - [While false](#while-false)
  - [While true](#while-true)
  - [Let assignment](#let-assignment)
  - [Assignment](#assignment)
  - [Return](#return)
- [Your type checker](#your-type-checker)
  - [Command sequence](#command-sequence-1)
  - [Operands for i32 types](#operands-for-i32-types)
  - [Operands for boolean types](#operands-for-boolean-types)
  - [If true/false](#if-truefalse)
  - [While true/false](#while-truefalse)
  - [Let assignment i32](#let-assignment-i32)
  - [Let assignment bool](#let-assignment-bool)
  - [Assignment i32](#assignment-i32)
  - [Assignment bool](#assignment-bool)
  - [Return i32](#return-i32)
  - [Return bool](#return-bool)
  - [Void (none)](#void-none)
- [Your borrrow checker](#your-borrrow-checker)
- [Your LLVM backend](#your-llvm-backend)
  - [Comparison](#comparison)
- [Overal course goals and learning outcomes.](#overal-course-goals-and-learning-outcomes)

## Your repo

- Link to your repo here: https://github.com/dynematic/d7050e

## Your syntax

- Give an as complete as possible EBNF grammar for your language

- Give an example that showcases all rules of your EBNF. The program should "do" something as used in the next excercise.

- If you support pointers, make sure your example covers pointers as well.

- Compare your solution to the requirements (as stated in the README.md). What are your contributions to the implementation.

Program
```ebnf
    : FunctionDec+
    ;
```
FunctionDec
```ebnf
    : "fn" Id "(" [ "," ], Params ")" "->" Type "{" Statement "}"
    | "fn" Id "(" [ "," ], Params ")" "{" Statement "}"
    ;
```
Params
```ebnf
    : "Id" ":" Type
    ;
```
Statement
```ebnf
    : "let" Id ":" Type AssignOp Expr ";"
    | "if" Log "{" Statement "}"
    | "while" Log "{" Statement "}"
    | "return" Expr ";"
    | Expr ";"
    ;
```
LogOp
```ebnf
    : "&&"
    | "||"
    | "!" 
    ;
```
CondOp
```ebnf
    : ">"
    | "<"
    | "=="
    | "!="
    ;
```
AssignOp
```ebnf
    : "="
    | "+="
    | "-="
    | "/="
    | "*="
    ;
```
SumOp
```ebnf
    : "+"
    | "-"
    ;
```
FactorOp
```ebnf
    : "*"
    | "/"
    ;
```
Type
```ebnf
    : "i32"
    | "bool"
    ;
```
Term
```ebnf
    : Num
    | Id
    | Bool
    | Function
    | "(" Expr ")"
    ;
```
Num
```ebnf
    : [ "-" ], Digit
    ;
```

Id
```ebnf
    : Letter , { Letter | Digit }, - White_space
```
Bool
```ebnf
    : "true"
    | "false"
    ;
```
Function
```ebnf
    : Id "(" [ "," ], Expr ")"
    ;
```
Letter
```ebnf
    : "A" | "B" | "C" | "D" | "E" | "F" | "G"
    | "H" | "I" | "J" | "K" | "L" | "M" | "N"
    | "O" | "P" | "Q" | "R" | "S" | "T" | "U"
    | "V" | "W" | "X" | "Y" | "Z" | "a" | "b"
    | "c" | "d" | "e" | "f" | "g" | "h" | "i"
    | "j" | "k" | "l" | "m" | "n" | "o" | "p"
    | "q" | "r" | "s" | "t" | "u" | "v" | "w"
    | "x" | "y" | "z" 
    ;
```
Digit
```ebnf
    : "0" | "1" | "2" | "3" | "4" | "5" | "6" | "7" | "8" | "9" 
    ;
```
White_space
```ebnf
    : ? White_space characters ? 
    ;
```

Showcase
```rust
fn test_i32(b: i32, c: i32) -> i32 {
    let a : i32 = 10 + 2 * 3;
    a = -1 - (1 - 1);
    return a + b;
}
fn test_bool(b: bool) -> bool {
    if b && true || false {
        let a : i32 = test_i32(1, 999);
        while (a < 5) {
            a = a + 1;
        }
        return false;
    }
    return (3 < 4);
}

fn main() {
    test_bool(true);
}
```
The parser has been implemented using an LR(1) parser generator, [lalrpop](https://github.com/lalrpop/lalrpop). It's a strong tool that offers your code to be DRY and have a readeble grammar from the beginning. It also offers macros which have been used to make the the code less repetetive, more readeble and to implement precedence. 

To write the parser the documention has been sufficient but examples from both the documentation and the lalrpop repo has been used.

Above descirbed EBNF and showcase defines a minimal subset of Rust, including
- Function declaration, with excplicit return type either void (NONE) or primitive type
- Primitive types are i32 and boolean
- let, assignment, if, while and return statements with explicit types
- Operands are divided into logical, conditionl and arithmetic operands with precedence
- Parantesized precedence and FactorOp has precedence over SumOp
- Assignment operands are seperated from above mentioned operands
- Location information for Error is not implemented
- Error recovery is not implemented

For furture development
- Rethink how assignment operands are handled in grammar
- Add if else statement
- Add support for String in Term
- Error handling, location information and recovery

## Your semantics

Structural Operational Semantics (SOS) for the compiler is described below in small step. 

Symbols: 
- σ, state
- σ', derived state
- ⇓, evaluates
- c, command
- x, variable
- e, expression

Expression can be either a
- b, boolean 
- n, number 
- f, function 

### Function sequence
```math
<f, σ>⇓σ'
```

```rust
fn main() {
    test();
}
```
The function call moves the current state, σ to the new derived state σ' in the function, test()'s context. A function can evaluate void (none), i32 and bool.

### Command sequence
```math
\frac{<c1,σ> ⇓ σ' <c2,σ'> ⇓ σ''}{<c1;c2,σ> ⇓ σ''}
```

```rust
let x : i32 = 2;
test();
```
A sequence is a composition of commands where the first command is first executed then the second command. Otherwise the intermediate step, σ' would be lost.

### Operands for arithmetic expressions
```math
\frac{<e1,σ> ⇓ n1 <e2, σ> ⇓ n2}{<e1 + e2, σ> ⇓ n1 \text{ } plus \text{ } n2}
```

plus is addition as implemented in the computer. Other arithmetic operands imlemented are
- -, minus
- *, multiplication
- /, division
- ">", greather than
-  "<", less than
- "==", equal
- "!=", not equal

```rust
(2 + 4) * 3;
1 < 2; // Is valid!
```

### Operands for boolean expressions
```math
\frac{<e1,σ> ⇓ b1 <e2, σ> ⇓ b2}{<e1 == e2, σ> ⇓ b1 \text{ } equal \text{ } b2}
```

how equal is evaluated is up to the computer. Other boolean operands implemented are

- "==", equal
- "!=", not equal
- "&&", and
- "||", or
```rust
2 != 3 || true;
1 != true; // Will panic!()
```


### If true
```math
\frac{<b, σ> ⇓ true <c1, σ> ⇓ σ'}{<if \text{ } b \text{ } then \text{ } c1> ⇓ σ}
```

then the state, $σ$ will change immediately to $σ'$.

### If false
```math
\frac{<b, σ> ⇓ false <c1, σ> ⇓ σ''}{<if \text{ } b \text{ } then \text{ } c1> ⇓ σ}
```

the state, $σ'$ will be changed at the second command to $σ''$.

### While false
```math
\frac{<b,σ> ⇓ false}{<while \text{ } b \text{ } do \text{ } c,σ> ⇓ σ}
```

### While true
```math
\frac{<b, σ> ⇓ true <c,σ> ⇓ σ' <while \text{ } b \text{ } do \text{ } c, σ'> ⇓ σ''}{<while \text{ } b \text{ } do \text{ } c,σ> ⇓ σ''}
```

```rust
let a : i32 = 0;
while a < 5 {
    a = a + 1;
}
```

then $σ'$ is the result of the current state, $σ$ and the last state $σ''$ is the result of $σ'$.

### Let assignment
```math
\frac{<x, σ>⇓<let \text{ } x:=e, σ>⇓σ'}{<let \text{ } x := e, σ> ⇓ σ[x := e]}
```
```rust
let a : i32 = 1;
let b : bool = true;
let c : i32 = test(); // c will be assigned whaterver test() returns
```
For let commands,  if the variable is already declared the state σ will be changed to the one of the second command, state σ', the variable will be lost. Otherwise the variable will be assigned to the state, σ.

### Assignment
```math
\frac{}{<x := e, σ> ⇓ σ[x := e]}
```

```rust
x = 1;
x = true;
x = test();
```
(OBS the interpreter does not check types, above assignments are valid in the interpreter)

For assignment, when the variable is assigned to a number, boolean or function the variable and corresponding rhs is moved to the same state.



### Return
```math
\frac{<f, σ>⇓σ}{<return : e, σ> ⇓ σ[return : e]}
```

```rust
fn main() -> i32 {
    return 1 + 1;
}
```
At explicit return the expression is moved to the state of the function.

Currently, the interpreter executes as above SOS describes. The interpretern does not type check e.g.
```rust
fn main() {
    return 1 + 1;
}
```
where an i32 is returned to a void (none) type declared funtion. It's valid in the interpreter. 

Meanwhile there are cases where the interpreter will panic!() e.g. return x + true or trying to assign an undeclared variable.

The interpreter has been implented with scopes and context in mind, this is described above in the SOS with state, σ. E.g. variables declared inside a if block/scope (state) are not accesible outside of that block/scope. 
```rust
if true {
    let a : i32 = 0;
}
a = 1; // Will panic!()
```
```rust
let a : i32 = 0;
if true {
    a = 1; // Will NOT panic!()
}
```
Each function has a unique context with a collection of scopes. The way context and scope have been implemented may be a bit off to the SOS rules of rust but not off according to above described SOS rules.

## Your type checker

Types are now defined as
- i32
- bool
- none


### Command sequence
```math
\frac{<c1,σ> ⇓ σ' <c2,σ'> ⇓ σ''}{<c1;c2,σ> ⇓ σ''}
```

as described in the SOS rules of the interpreter.


### Operands for i32 types
```math
\frac{<e1,σ> ⇓ i32 <e2, σ> ⇓ i32}{<e1 + e2, σ> ⇓ i32}
```

To operate these operands the expression need to evaluate to i32. Other arithmetic operands imlemented are
- -, minus
- *, multiplication
- /, division
- ">", greather than
-  "<", less than
- "==", equal
- "!=", not equal

```rust
1 + 1;
1 + true; //TypeError
```

### Operands for boolean types
```math
\frac{<e1,σ> ⇓ bool <e2, σ> ⇓ bool}{<e1 == e2, σ> ⇓ bool}
```

To operate these operands the expression need to evaluate to boolean. Other boolean operands implemented are

- "==", equal
- "!=", not equal
- "&&", and
- "||", or

```rust
true || false;
1 || true; //TypeError
```

### If true/false
```math
\frac{<e, σ> ⇓ bool <c1, σ> ⇓ σ'}{<if \text{ } bool \text{ } then \text{ } c1> ⇓ σ}
```

```rust
if false {
    test(); // This will execute 
}
```

### While true/false
```math
\frac{<e, σ> ⇓ bool <c,σ> ⇓ σ' <while \text{ } bool \text{ } do \text{ } c, σ'> ⇓ σ''}{<while \text{ } bool \text{ } do \text{ } c,σ> ⇓ σ''}
```

For both if and while commands their block/scope are evluated as long as their condition is evaluted to the type boolean. This differ from the interpreter which only executes the block/scope if the condition is true.


### Let assignment i32
```math
\frac{<x, σ>⇓i32<let \text{ } x:=i32, σ>⇓σ'}{<let \text{ } x := i32, σ> ⇓ σ[x := i32]}
```
```rust
let a : i32 = 1;
let b : i32 = test(); // test() must return type i32 otherwise Error
```
### Let assignment bool
```math
\frac{<x, σ>⇓bool<let \text{ } x:=bool, σ>⇓σ'}{<let \text{ } x := bool, σ> ⇓ σ[x := bool]}
```
```rust
let a : bool = true;
let b : bool = test(); // test() must return type bool otherwise Error
```
As previously mentioned, let assignment are unique declerations. Thus, the same variable can not be declared twice in the same scope. Here, the given return type of the variable is checked with rhs evluated type.

### Assignment i32
```math
\frac{<x, σ> ⇓ i32<e, σ> i32}{<x := i32, σ> ⇓ σ[x := i32]}
```

```rust
x = (1 + 1) * 3;
```
### Assignment bool
```math
\frac{<x, σ> ⇓ bool<e, σ> bool}{<x := bool, σ> ⇓ σ[x := bool]}
```

```rust
x = true != false;
```
For any assigment, the variable typed stored with the rhs evaluated type is checked.



### Return i32
```math
\frac{<f, σ>⇓σ}{<return : i32, σ> ⇓ σ[return : i32]}
```

```rust
fn main() -> i32 {
    return 1 + 1;
    return true; // Error
}
```
The function is declared to return a type i32.


### Return bool
```math
\frac{<f, σ>⇓σ}{<return : bool, σ> ⇓ σ[return : bool]}
```

```rust
fn main() -> bool {
    return true && true;
    return 3; // Error
}
```
The function is declared to return a type boolean.

### Void (none)
```math
\frac{<f, σ>⇓σ}{<none, σ> ⇓ σ[none]}
```

```rust
fn main() {
    let a : i32 = 1 + 1;
    return a; //ReturnError
}
```

If the function does not explicit return then the type none is returned.

Missing in the type checker are fully checking that a function returns, see example
```rust
fn test() -> i32 {
    if true {
        return 1 - 1;
    }
    // Error? This block returns none, with if command, it might not return!
}
```

SOS rules of the type checker that has not been described above are the type checking of function parameter, argument and length.

TO DISCUSS:
Alternative answer would be to defined the SOS rules for functions as
```math
<f, i32>⇓i32, <f, bool>⇓bool, <f, none>⇓none. 
```

The type checker will return a custom error with sufficient information except location e.g.
```rust
enum Error {
    TypeError(Type, Type, Expr),
    ReturnError(Type, Type, String),
    ...
}
```


In the future the type checking can be expanded to handle mutability and memory reference. Also, to implement more sophisticated error handling as error chaining (multiple error) and location.


## Your borrrow checker

- Any borrow may only last for the lifetime of the owners scope.
- One mutably borrow at a time. E.g. function parameter references may not reference to the same value.
- One or more immutably borrows at a time. E.g. to reference an immutable value is possible at any time.
- You cannot borrow a mutably and immutably at the same time.

For the dev, do not try to do multiple simultaneous mutable borrows. Store your future changes in a separate location and then apply them.


The end goal of the borrow checker is to prevent data-racing. Data-racing is when two or more pointers are pointing to the same memory location where one ore more pointers are writing to the location. The operations are not syncronizhed.
## Your LLVM backend

- Let your backend produces LLVM-IR for your example program.
```llvm

; ModuleID = 'main'
source_filename = "main"
target datalayout = "e-m:e-i64:64-f80:128-n8:16:32:64-S128"

define i32 @test_i32(i32, i32) {
entry:
  %a = alloca i32
  %c = alloca i32
  %b = alloca i32
  store i32 %0, i32* %b
  store i32 %1, i32* %c
  store i32 15, i32* %a
  store i32 1, i32* %a
  %a1 = load i32, i32* %a
  %b2 = load i32, i32* %b
  %Sum = add i32 %a1, %b2
  ret i32 %Sum
}

define i1 @test_bool(i1) {
entry:
  %b = alloca i32
  store i1 %0, i32* %b
  %b1 = load i32, i32* %b
  br i32 %b1, label %if, label %cont

if:                                               ; preds = %entry
  %a = alloca i32
  %test_i32 = call i32 @test_i32(i32 1, i32 999)
  store i32 %test_i32, i32* %a
  %a2 = load i32, i32* %a
  %Lt = icmp slt i32 %a2, 5
  br i1 %Lt, label %while, label %cont3

cont:                                             ; preds = %cont3, %entry
  %iftmp = phi i32 
  ret i1 true

while:                                            ; preds = %while, %if
  %a4 = load i32, i32* %a
  %Sum = add i32 %a4, 1
  store i32 %Sum, i32* %a
  br i1 %Lt, label %while, label %cont3

cont3:                                            ; preds = %while, %if
  %whiletmp = phi i32 
  ret i1 true
  br label %cont
}

define void @main() {
entry:
  %test_bool = call i1 @test_bool(i1 true)
}
```

- Allocation are done to store a variable on the stack, such that LLVM later can access the variable in the memory using given pointer. This is done when storing parameters, arguments, variables etc.
  
- Phi nodes are supposed to be used in if and while statements in my implementation. Because of the way I have built the block functionality it gave me a headech to change at this late of a stage- Therefor I will explain phi nodes. They way I interpreted them! 
  - Phi nodes are used to handle different blocks in e.g. a if block. There are two blocks and because SSA (Static single assignment) a variable cannot be assigned more than once.

```rust
if ... {
    a.0 = 1;
} else {
    a.1 = 2;
}
...
```
Since LLVM uses the variable with highest index, in the example above a.1 = 2. What if the condition were true? Then we want a.0 = 1! This is the problem with SSA in LLVM and therefor phi nodes are used e.g.
```rust
if ... {
    phi.if_block
} else {
    phi.else_block
}
phi.cont_block
...
```
### Comparison
- No PassManager has been used or other optimization, except some [#inline] somewere and verify.

- LLVM is a good wrapper to the course as I will talk more about but I have been introduced to
  - SSA form, it's hiccups and how we bypass them
  - The concept of unique I would like to discuss
  - LLVM optimization as the example of PassManager, verify, more to e discovered...
  - By using the It's a New Kind of Wrapper for Exposing LLVM (safely) I have been introduced to some LLVM-API during implementation. 

## Overal course goals and learning outcomes.

- Lexical analysis, syntax analysis, and translation into abstract syntax.
  - As I took the decesion to use a parser generator, lalrpop the lexical analysis and syntax analysis has been simplified. Lalrpop made it easy to understand lexical tokenizing and it was a strong tool for syntax analysis e.g. precedence or paranthesis handling. Then it all boils down to the building block of my compiler, the AST which I have found it hard to get 'perfect' but has been fun to work with and given me a better understanding after each iteration.

- Regular expressions and grammars, context-free languages and grammars, lexer and parser generators. [Nom is lexer/parser library (and replaces the need for a generator, while lalr-pop is a classical parser generator)]
    - As mentioned above, first tokenizing the input to produce terminals and then recognizing the terminals and executing the code. Thus, lalrpop is a canonical LR parser or LR(1) parser which handles my context-free language e.g. arethmetic expressions. By using lalrpop the EBNF is basically given.

- Identifier handling and symbol table organization. Type-checking, logical inference systems. [SOS is a logical inference system]
  - Type checking gave me a realisation that my interpreter may have fault in how it executes. Other than that has type checking given me some more insight in the ownership model of rust and implemented my own interpretation, described in the SOS.
  - SOS describes my compiler language in a more formal way, how my compiler works! But is has been a bit tricky to define the SOS and I expect some scemantics may be wrong. Identifier handling and symbol table is trivial to defining the SOS.
  - At this point I excpect the interpreter not to fail.

- LLVM
  - This is the last step of my compiler, now we are generating actual machine code. It has linked this course with microcomputing which I feel has been rewarding. To think more of e.g. how boolean are represented in machine code and the problems surrounding SSA form. To grasp LLVM you could have had a seperate course but it wraps up the compiler you have written and produce during the course to something useful.

Comment on additional things that you have experienced and learned throughout the course.

To get the AST "perfect" is hard and needs a lot of interations. Therefor a complete or class complete parser could have been given. Focus on 
- Interpretation, to understand how will your compiler execute, how does RUST execute?
- Type checking, what type rules do RUST enforce?
- Borrow checking, it slaps you in the face but you gradually get a more understanding of it and why it exists in the first place.
- LLVM, a wrapper for the course that could have gotten more love to show it's twits and bits.

In all I have gotten a understanding of how a compiler works and the formality to describe a language. A more in-depth understanding of RUST than I previously knew. It has given me the a good stepping stone to be able to build and define my own language of a compiler.